#include <optixu/optixpp_namespace.h>
#include <sutil.h>
#include <stdlib.h>
#include <string.h>

using namespace optix;

const char* const ptxpath( const std::string& target, const std::string& base )
{
  static std::string path;
  path = std::string(sutilSamplesPtxDir()) + "/" + target + "_generated_" + base + ".ptx";
  return path.c_str();
}

int main( int argc, char** argv )
{
  try {
    optix::Context context = Context::create();
    context->setRayTypeCount( 1 );
    context->setEntryPointCount( 1 );

    Buffer buffer = context->createBuffer( RT_BUFFER_OUTPUT, RT_FORMAT_FLOAT4, 512, 512);
    context["output_buffer"]->set(buffer);

    Program exceptionProgram = context->createProgramFromPTXFile(ptxpath("helsinki", "BiDirCamera.cu"), "exception");
    Program ray_gen_program = context->createProgramFromPTXFile(ptxpath( "helsinki", "BiDirCamera.cu" ), "pathtrace_camera");
    ray_gen_program["sampleLightPath"]          ->set(context->createProgramFromPTXFile(ptxpath( "helsinki", "BiDirCamera.cu" ), "sampleLightPath_f"));
    ray_gen_program["sampleEye"]                ->set(context->createProgramFromPTXFile(ptxpath( "helsinki", "BiDirCamera.cu" ), "sampleEye_f"));

    context->setRayGenerationProgram(0, ray_gen_program);
    context->setExceptionProgram(0, exceptionProgram);
    context->validate();
    context->compile();

    RTsize buffer_width, buffer_height;
    buffer->getSize( buffer_width, buffer_height );

    context->launch(0, static_cast<unsigned int>(buffer_width), static_cast<unsigned int>(buffer_height));
  } catch( Exception& e ){
    sutilReportError( e.getErrorString().c_str() );
    exit(1);
  }

  return 0;
}
